/*	imheader - Text Input Method Header
 *
 *  standard_im.h - 
 *
 *	Authors: YE Nan <nan.ye@orange-ftgourp.com>,
 *           LUO Lei <Srella@163.com>
 *
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 * 	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */

//file API definition of the Standard Input Method interface.


#ifndef __SIMP_IM_ENGINE_H
#define __SIMP_IM_ENGINE_H

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

typedef enum _IME_Bool {
	IME_NONE  = -1,
	IME_FALSE = 0,
	IME_TRUE  = 1
} IME_Bool;

typedef enum _IME_Status {
	IME_PREEDIT,
	IME_NORMAL,
	IME_ASSOCIATE,
	IME_PUNCT
} IME_Status;


typedef struct _IME_InputMethod {
	const char *	uuid;
	const char *	encoding;
	const char *	name;
	const char *	icon;
	const char *	languages;
	const char *	help;
	const char *	copyright;
	const char *	version;
	const int		capability_mask;
} IME_InputMethod;

/*IMEngine IMHelper capability*/
#define	CAP_NUM_INPUT					0x01
#define	CAP_TS_INPUT						0x02
#define	CAP_TS_FCITX_INPUT		0x04


#define FOR_PRE_CAND			1
#define FOR_CAND					2

typedef struct _IME_InputContext {
	int				id;
	const char *	uuid;
	const char *	charset;
	void *			private_data;
} IME_InputContext;


typedef struct _IME_Helper_Info{
	const char *	uuid;
	const char *	encoding;
	const char *	name;
	const char *	description;
	const char *	icon;
	const int		capability_mask;
} IME_Helper_Info;


typedef struct _IME_KeyEvent {
	int	code;
	int	unicode;
	int	mask;
} IME_KeyEvent;


typedef enum _IME_AttributeType {
	IME_ATTR_NONE,
	IME_ATTR_DECORATION,
	IME_ATTR_FOREGROUND_RGB,
	IME_ATTR_BACKGROUND_RGB
} IME_AttributeType;


#define IME_DECORATION_NONE			0
#define IME_DECORATION_UNDERLINE	1
#define IME_DECORATION_HIGHLIGHT	2
#define IME_DECORATION_REVERSE		4

#define IME_RGB_COLOR(RED,GREEN,BLUE)	((int)(((RED)<<16) + ((GREEN)<<8) + (BLUE)))
#define IME_RGB_COLOR_RED(COLOR)		((int)(COLOR>>16))
#define IME_RGB_COLOR_GREEN(COLOR)		((int)((COLOR>>8)&0x00ff))
#define IME_RGB_COLOR_BLUE(COLOR)		((int)(COLOR&0x0000ff))


typedef struct _IME_Attribute {
	IME_AttributeType	type;
	int					value;
	int					start;
	int					length;
} IME_Attribute;


typedef struct _IME_String {
	const char *	string;
	int				nr_attributes;
	IME_Attribute *	attributes;
} IME_String;


typedef struct _IME_Candidate {
	const char *	label;
	IME_String		content;
} IME_Candidate;


#define IME_NORMAL_PAGE	0
#define IME_FIRST_PAGE		1
#define IME_LAST_PAGE		2
#define IME_SINGLE_PAGE	3


typedef struct _IME_CandidatesList {
	int				focused_candidate;
	int				page_state;
	int				nr_candidates;
	const char *	title;
	IME_Candidate *	candidates;
} IME_CandidatesList;

#define IME_PROPERTY_ACTIVE   1
#define IME_PROPERTY_VISIBLE  2


typedef struct _IME_Functions {
	void		(*initialize)							(void);
	void		(*finalize)								(void);
	int			(*get_number_of_input_methods)			(void);
	IME_Bool	(*get_input_method_info)				(int index, IME_InputMethod * info);
	IME_Bool	(*create_input_context)					(IME_InputContext * ic);
	void		(*destroy_input_context)				(IME_InputContext * ic);
	IME_Bool	(*add_element)							(const IME_InputContext * ic, void * element);
	IME_Bool	(*direct_commit)							(const IME_InputContext * ic, void * element);
	IME_Bool	(*del_element)							(const IME_InputContext * ic);
	void		(*reset)								(const IME_InputContext * ic);
	void		(*select_preedit_candidate)				(const IME_InputContext * ic, int index);
	void		(*select_candidate)						(const IME_InputContext * ic, int index);
	void		(*previous_preedit_candidates_page)		(const IME_InputContext * ic);
	void		(*previous_candidates_page)				(const IME_InputContext * ic);
	void		(*next_preedit_candidates_page)			(const IME_InputContext * ic);
	void		(*next_candidates_page)					(const IME_InputContext * ic);
	void		(*update_preedit_candidates_page_size)	(const IME_InputContext * ic, int size);
	void		(*update_candidates_page_size)			(const IME_InputContext * ic, int size);
	void		(*set_cand_num)				(int charnum, unsigned short selection);
} IME_Functions;


typedef struct _IMM_Functions {
	void	(*show_preedit_candidates_list)		(const IME_InputContext * ic);
	void	(*show_candidates_list)				(const IME_InputContext * ic);
	void	(*hide_all_candidates_list)			(const IME_InputContext * ic);
	void	(*update_preedit_candidates_list)	(const IME_InputContext * ic, const IME_CandidatesList * candidates);
	void	(*update_candidates_list)			(const IME_InputContext * ic, const IME_CandidatesList * candidates);
	void	(*commit_string)					(const char * string);
	IME_Status (*get_input_method_status)		(void);
	void (*set_cand_num_to_engine)		(int charnum, unsigned short selection);
} IMM_Functions;


typedef struct _IMH_Functions {
	void		(*initialize)					(const char * name);	
	void		(*finalize)						(void);
	IME_Bool	(*process_helper_key_event)		(const IME_KeyEvent * key);
	void		(*show_preedit_candidates_list)	(const IME_CandidatesList * list);
	void		(*show_candidates_list)			(const IME_CandidatesList * list);
	void		(*hide_all_candidates_list)		(void);
	void		(*notify_ime_status)			(IME_Status status);
	void		(*notify_numeric_display)		(IME_Bool numeric_display);
	void		(*notify_client_size)			(int x, int y, int width, int height);
	int   (*get_number_of_helper)		(void);
	int   (*get_helper_info)		(int index, IME_Helper_Info * info);
} IMH_Functions;


IME_Bool	ime_register_module		(const IMM_Functions * imm, IME_Functions * ime);
IME_Bool	imh_register_module		(const IMM_Functions * imm,	 IMH_Functions * imh);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* __SIMP_IM_ENGINE_H */
